@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/createpdf.py $
-#$LastChangedDate: 2010-10-06 18:21:41 -0300 (Wed, 06 Oct 2010) $
-#$LastChangedRevision: 2396 $
+#$URL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/createpdf.py $
+#$Date: 2012-05-25 09:56:14 -0300 (Fri, 25 May 2012) $
+#$Revision: 2497 $
 
 # See LICENSE.txt for licensing terms
 
@@ -50,13 +50,16 @@ import os
 import tempfile
 import re
 import string
+import config
+import logging
+from cStringIO import StringIO
 from os.path import abspath, dirname, expanduser, join
 from urlparse import urljoin, urlparse, urlunparse
 from copy import copy, deepcopy
 from optparse import OptionParser
-import logging
+from pprint import pprint
+from xml.sax.saxutils import unescape, escape
 
-from docutils.languages import get_language
 import docutils.readers.doctree
 import docutils.core
 import docutils.nodes
@@ -64,8 +67,10 @@ from docutils.parsers.rst import directives
 from docutils.readers import standalone
 from docutils.transforms import Transform
 
-import pygments_code_block_directive # code-block directive
-import oddeven_directive 
+try:
+    from roman import toRoman
+except ImportError:
+    from docutils.utils.roman import toRoman
 
 from reportlab.platypus import *
 from reportlab.platypus.doctemplate import IndexingFlowable
@@ -75,63 +80,48 @@ from reportlab.pdfbase.pdfdoc import PDFPageLabel
 #from reportlab.lib.units import *
 #from reportlab.lib.pagesizes import *
 
-from flowables import * # our own reportlab flowables
-from sinker import Sinker
-import flowables
-from image import MyImage, missing
-
-from aafigure_directive import Aanode
-import counter_role
-
-from log import log, nodeid
-from pprint import pprint
-
-from smartypants import smartyPants
-
-from roman import toRoman
+from rst2pdf import counter_role, oddeven_directive
+from rst2pdf import pygments_code_block_directive # code-block directive
+from rst2pdf import flowables
+from rst2pdf.flowables import * # our own reportlab flowables
+from rst2pdf.sinker import Sinker
+from rst2pdf.image import MyImage, missing
+from rst2pdf.aafigure_directive import Aanode
+from rst2pdf.log import log, nodeid
+from rst2pdf.smartypants import smartyPants
+from rst2pdf import styles as sty
+from rst2pdf.nodehandlers import nodehandlers
+from rst2pdf.languages import get_language_available
+from rst2pdf.opt_imports import Paragraph, BaseHyphenator, PyHyphenHyphenator, \
+    DCWHyphenator, sphinx as sphinx_module, wordaxe
 
 # Small template engine for covers
 # The obvious import doesn't work for complicated reasons ;-)
-import tenjin
-to_str=tenjin.helpers.generate_tostrfunc('utf-8')
-escape=tenjin.helpers.escape
-templateEngine=tenjin.Engine()
+from rst2pdf import tenjin
+to_str = tenjin.helpers.generate_tostrfunc('utf-8')
+escape = tenjin.helpers.escape
+templateEngine = tenjin.Engine()
 
 def renderTemplate(tname, **context):
-  context['to_str']=to_str
-  context['escape']=escape
+  context['to_str'] = to_str
+  context['escape'] = escape
   return templateEngine.render(tname, context)
 
-# Is this really the best unescape in the stdlib for '&amp;' => '&'????
-from xml.sax.saxutils import unescape, escape
-
-import config
-
-from cStringIO import StringIO
-
 #def escape (x,y):
 #    "Dummy escape function to test for excessive escaping"
 #    return x
-from utils import log
-import styles as sty
-
-from opt_imports import Paragraph, BaseHyphenator, PyHyphenHyphenator, \
-                         DCWHyphenator, sphinx as sphinx_module, wordaxe
-
-from nodehandlers import nodehandlers
 
 numberingstyles={ 'arabic': 'ARABIC',
                   'roman': 'ROMAN_UPPER',
                   'lowerroman': 'ROMAN_LOWER',
                   'alpha':  'LETTERS_UPPER',
                   'loweralpha':  'LETTERS_LOWER' }
-                 
 
 
 class RstToPdf(object):
 
     def __init__(self, stylesheets=[],
-                 language='en',
+                 language='en_US',
                  header=None,
                  footer=None,
                  inlinelinks=False,
@@ -149,7 +139,7 @@ class RstToPdf(object):
                  real_footnotes=False,
                  def_dpi=300,
                  show_frame=False,
-                 highlightlang='python', #This one is only used by sphinx
+                 highlightlang='python', # this one is only used by Sphinx
                  basedir=os.getcwd(),
                  splittables=False,
                  blank_first_page=False,
@@ -157,6 +147,8 @@ class RstToPdf(object):
                  custom_cover='cover.tmpl',
                  floating_images=False,
                  numbered_links=False,
+                 section_header_depth=2,
+                 raw_html=False
                  ):
         self.debugLinesPdf=False
         self.depth=0
@@ -164,18 +156,8 @@ class RstToPdf(object):
         self.blank_first_page=blank_first_page
         self.splittables=splittables
         self.basedir=basedir
-        self.language = language
-        try:
-            get_language (self.language)
-        except ImportError:
-            try:
-                language = self.language.split('_', 1)[0]
-                get_language (language)
-                self.language = language
-            except ImportError:
-                log.warning("Can't load Docutils module "\
-                    "for language %s or %s", self.language, language)
-                self.language = 'en'
+        self.language, self.docutils_language = get_language_available(
+            language)[:2]
         self.doc_title = ""
         self.doc_title_clean = ""
         self.doc_subtitle = ""
@@ -193,13 +175,13 @@ class RstToPdf(object):
             self.PATH = abspath(dirname(sys.executable))
         else:
             self.PATH = abspath(dirname(__file__))
-            
-            
+
+
         self.font_path=font_path
         self.style_path=style_path
         self.def_dpi=def_dpi
         self.loadStyles(stylesheets)
-            
+
         self.docutils_languages = {}
         self.inlinelinks = inlinelinks
         self.breaklevel = breaklevel
@@ -218,7 +200,9 @@ class RstToPdf(object):
         self.def_dpi = def_dpi
         self.show_frame = show_frame
         self.numbered_links = numbered_links
+        self.section_header_depth = section_header_depth
         self.img_dir = os.path.join(self.PATH, 'images')
+        self.raw_html = raw_html
 
         # Sorry about this, but importing sphinx.roles makes some
         # ordinary documents fail (demo.txt specifically) so
@@ -242,18 +226,12 @@ class RstToPdf(object):
             if self.language:
                 self.styles.languages.append(self.language)
                 self.styles['bodytext'].language = self.language
-
+            else:
+                self.styles.languages.append('en_US')
+                self.styles['bodytext'].language = 'en_US'
         # Load the docutils language modules for all required languages
         for lang in self.styles.languages:
-            try:
-                self.docutils_languages[lang] = get_language(lang)
-            except ImportError:
-                try:
-                    self.docutils_languages[lang] = \
-                         get_language(lang.split('_', 1)[0])
-                except ImportError:
-                    log.warning("Can't load Docutils module \
-                        for language %s", lang)
+            self.docutils_languages[lang] = get_language_available(lang)[2]
 
         # Load the hyphenators for all required languages
         if wordaxe is not None:
@@ -265,8 +243,8 @@ class RstToPdf(object):
                     except Exception:
                         # hyphenators may not always be available or crash,
                         # e.g. wordaxe issue 2809074 (http://is.gd/16lqs)
-                        log.warning("Can't load wordaxe DCW hyphenator "
-                        "for German language, trying Py hyphenator instead")
+                        log.warning("Can't load wordaxe DCW hyphenator"
+                        " for German language, trying Py hyphenator instead")
                     else:
                         continue
                 try:
@@ -286,12 +264,12 @@ class RstToPdf(object):
 
         self.pending_targets=[]
         self.targets=[]
-        
+
     def loadStyles(self, styleSheets=None ):
-        
+
         if styleSheets is None:
             styleSheets=[]
-        
+
         self.styles = sty.StyleSheet(styleSheets,
                                      self.font_path,
                                      self.style_path,
@@ -318,8 +296,8 @@ class RstToPdf(object):
     def text_for_label(self, label, style):
         """Translate text for label."""
         try:
-            text = self.docutils_languages[self.style_language(style)]\
-                    .labels[label]
+            text = self.docutils_languages[
+                        self.style_language(style)].labels[label]
         except KeyError:
             text = label.capitalize()
         return text
@@ -327,8 +305,8 @@ class RstToPdf(object):
     def text_for_bib_field(self, field, style):
         """Translate text for bibliographic fields."""
         try:
-            text = self.docutils_languages[self.style_language(style)]\
-                    .bibliographic_fields[field]
+            text = self.docutils_languages[
+                        self.style_language(style)].bibliographic_fields[field]
         except KeyError:
             text = field
         return text + ":"
@@ -336,13 +314,12 @@ class RstToPdf(object):
     def author_separator(self, style):
         """Return separator string for authors."""
         try:
-            sep = self.docutils_languages[self.style_language(style)]\
-                    .author_separators[0]
+            sep = self.docutils_languages[
+                    self.style_language(style)].author_separators[0]
         except KeyError:
             sep = ';'
         return sep + " "
 
-
     def styleToTags(self, style):
         '''Takes a style name, returns a pair of opening/closing tags for it, like
         "<font face=helvetica size=14 color=red>". Used for inline
@@ -366,13 +343,12 @@ class RstToPdf(object):
             if s.underline:
                 r1.append('<u>')
                 r2.insert(0,'</u>')
-            
+
             return [''.join(r1), ''.join(r2)]
         except KeyError:
             log.warning('Unknown class %s', style)
             return None
 
-        
 
     def styleToFont(self, style):
         '''Takes a style name, returns a font tag for it, like
@@ -387,7 +363,7 @@ class RstToPdf(object):
             if bc:
                 r.append('backColor="#%s"' % bc.hexval()[2:])
             if s.trueFontSize:
-                r.append('size="%d"'%s.fontSize) 
+                r.append('size="%d"'%s.fontSize)
             r.append('>')
             return ''.join(r)
         except KeyError:
@@ -420,14 +396,14 @@ class RstToPdf(object):
         else:
             start = 1
 
-        if node.parent.get('bullet') or \
-            isinstance(node.parent, docutils.nodes.bullet_list):
-            b = node.parent.get('bullet','*')
+        if node.parent.get('bullet') or isinstance(
+                node.parent, docutils.nodes.bullet_list):
+            b = node.parent.get('bullet', '*')
             if b == "None":
                 b = ""
             t = 'bullet'
 
-        elif node.parent.get('enumtype')=='arabic':
+        elif node.parent.get('enumtype') == 'arabic':
             b = str(node.parent.children.index(node) + start) + '.'
 
         elif node.parent.get('enumtype') == 'lowerroman':
@@ -521,9 +497,9 @@ class RstToPdf(object):
             content=[XXPreformatted(text, style)],
             mode=self.fit_mode, style=style)
 
-    def createPdf(self, text=None, 
-                  source_path=None, 
-                  output=None, 
+    def createPdf(self, text=None,
+                  source_path=None,
+                  output=None,
                   doctree=None,
                   compressed=False,
                   # This adds entries to the PDF TOC
@@ -541,7 +517,7 @@ class RstToPdf(object):
                            'footer': self.footer,
                            'endnotes': [],
                            'extraflowables': []}
-                           
+
         self.pending_targets=[]
         self.targets=[]
 
@@ -550,7 +526,7 @@ class RstToPdf(object):
         if doctree is None:
             if text is not None:
                 if self.language:
-                    settings_overrides={'language_code': self.language}
+                    settings_overrides={'language_code': self.docutils_language}
                 else:
                     settings_overrides={}
                 self.doctree = docutils.core.publish_doctree(text,
@@ -570,7 +546,7 @@ class RstToPdf(object):
             self.doctree.walk(snf)
             srf = SectRefExpander(self.doctree, snf.sectnums)
             self.doctree.walk(srf)
-            
+
         elements = self.gen_elements(self.doctree)
 
         # Find cover template, save it in cover_file
@@ -621,8 +597,8 @@ class RstToPdf(object):
             # Handle images with alignment more like in HTML
             new_elem=[]
             for i,e in enumerate(elements[::-1]):
-                if isinstance (e, MyImage) and e.image.hAlign != 'CENTER'\
-                and new_elem:
+                if (isinstance (e, MyImage) and e.image.hAlign != 'CENTER'
+                        and new_elem):
                     # This is an image where flowables should wrap
                     # around it
                     popped=new_elem.pop()
@@ -670,7 +646,7 @@ class RstToPdf(object):
                 # first pass in the case of ###Total###, then we
                 # make a new forced two-pass build. This is broken.
                 # conceptually.
-                
+
                 if getattr(self, 'mustMultiBuild', False):
                     # Force a multibuild pass
                     if not isinstance(elements[-1],UnhappyOnce):
@@ -686,8 +662,8 @@ class RstToPdf(object):
                             # Add it to the pile
                             #if not isinstance (e, MySpacer):
                             fnPile.append(e)
-                        elif getattr(e, '_atTop', False) or \
-                            isinstance (e, (UnhappyOnce, MyPageBreak)):
+                        elif getattr(e, '_atTop', False) or isinstance(
+                                e, (UnhappyOnce, MyPageBreak)):
                             if fnPile:
                                 newStory.append(Sinker(fnPile))
                             newStory.append(e)
@@ -702,7 +678,7 @@ class RstToPdf(object):
                     continue
 
 
-                
+
                 break
             except ValueError, v:
                 # FIXME: cross-document links come through here, which means
@@ -737,7 +713,7 @@ class FancyDocTemplate(BaseDocTemplate):
             # Notify TOC entry for headings/abstracts/dedications.
             level, text = flowable.level, flowable.text
             parent_id = flowable.parent_id
-            node = flowable.node  
+            node = flowable.node
             pagenum = setPageCounter()
             self.notify('TOCEntry', (level, text, pagenum, parent_id, node))
 
@@ -790,7 +766,8 @@ class FancyDocTemplate(BaseDocTemplate):
                             self.afterFlowable(S[0])
                             doctemplate._addGeneratedContent(flowables,frame)
                         else:
-                            ident = "Splitting error(n==%d) on page %d in\n%s" % (n,self.page,self._fIdent(f,60,frame))
+                            ident = "Splitting error(n==%d) on page %d in\n%s" % (
+                                n, self.page, self._fIdent(f, 60, frame))
                             #leave to keep apart from the raise
                             raise LayoutError(ident)
                         del S[0]
@@ -798,17 +775,17 @@ class FancyDocTemplate(BaseDocTemplate):
                         flowables.insert(i,f)   # put split flowables back on the list
                 else:
                     if hasattr(f,'_postponed') and f._postponed > 4:
-                        ident = "Flowable %s%s too large on page %d in frame %r%s of template %r" % \
-                                (self._fIdent(f,60,frame),doctemplate._fSizeString(f),self.page, self.frame.id,
-                                        self.frame._aSpaceString(), self.pageTemplate.id)
+                        ident = "Flowable %s%s too large on page %d in frame %r%s of template %r" % (
+                            self._fIdent(f, 60, frame), doctemplate._fSizeString(f),self.page,
+                            self.frame.id, self.frame._aSpaceString(), self.pageTemplate.id)
                         #leave to keep apart from the raise
                         raise LayoutError(ident)
                     # this ought to be cleared when they are finally drawn!
                     f._postponed = 1
-                    mbe = getattr(self,'_multiBuildEdits',None)
+                    mbe = getattr(self, '_multiBuildEdits', None)
                     if mbe:
-                        mbe((delattr,f,'_postponed'))
-                    flowables.insert(0,f)           # put the flowable back
+                        mbe((delattr, f, '_postponed'))
+                    flowables.insert(0, f)           # put the flowable back
                     self.handle_frameEnd()
 
 
@@ -830,7 +807,7 @@ class PageCounter(Flowable):
 
     def drawOn(self, canvas, x, y, _sW):
         pass
-        
+
 flowables.PageCounter = PageCounter
 
 def setPageCounter(counter=None, style=None):
@@ -853,13 +830,13 @@ def setPageCounter(counter=None, style=None):
     else:
         ptext=unicode(_counter)
     return ptext
-    
+
 class MyContainer(_Container, Flowable):
     pass
 
 class UnhappyOnce(IndexingFlowable):
     '''An indexing flowable that is only unsatisfied once.
-    If added to a story, it will make multiBuild run 
+    If added to a story, it will make multiBuild run
     at least two passes. Useful for ###Total###'''
     _unhappy=True
     def isSatisfied(self):
@@ -867,7 +844,7 @@ class UnhappyOnce(IndexingFlowable):
             self._unhappy= False
             return False
         return True
-        
+
     def draw(self):
         pass
 
@@ -935,7 +912,7 @@ class HeaderOrFooter(object):
                 getattr(canv, 'sectNum', ''))
             text = smartyPants(text, smarty)
             return text
-        
+
         for i,e  in enumerate(elems):
             if isinstance(e, Paragraph):
                 text = replace(e.text)
@@ -955,7 +932,7 @@ class HeaderOrFooter(object):
                 even=self.replaceTokens([e.even,], canv, doc, smarty)[0]
                 elems[i]=OddEven(odd, even)
         return elems
-        
+
     def draw(self, pageobj, canv, doc, x, y, width, height):
         self.totalpages = max(self.totalpages, doc.page)
         items = self.prepared
@@ -1007,11 +984,11 @@ class FancyPage(PageTemplate):
                 return
             try:
                 w, h, kind = MyImage.size_for_node(dict(uri=uri, ), self.client)
-            except ValueError: 
+            except ValueError:
                 # Broken image, return arbitrary stuff
                 uri=missing
                 w, h, kind = 100, 100, 'direct'
-                
+
             pw, ph = self.styles.pw, self.styles.ph
             if self.client.background_fit_mode == 'center':
                 scale = min(1.0, 1.0 * pw / w, 1.0 * ph / h)
@@ -1025,7 +1002,7 @@ class FancyPage(PageTemplate):
                 # Do scale anyway
                 x, y = 0, 0
                 sw, sh = pw, ph
-                
+
             bg = MyImage(uri, sw, sh, client=self.client)
             self.image_cache[uri] = info = bg, x, y
         bg, x, y = info
@@ -1041,8 +1018,8 @@ class FancyPage(PageTemplate):
 
         global _counter, _counterStyle
 
-        self.tw = self.styles.pw - self.styles.lm -\
-            self.styles.rm - self.styles.gm
+        styles = self.styles
+        self.tw = styles.pw - styles.lm - styles.rm - styles.gm
         # What page template to use?
         tname = canv.__dict__.get('templateName',
                                   self.styles.firstTemplate)
@@ -1052,34 +1029,33 @@ class FancyPage(PageTemplate):
         doct = getattr(canv, '_doctemplate', None)
         canv._doctemplate = None # to make _listWrapOn work
 
-        if doc.page==1:
-            _counter=0
-            _counterStyle='arabic'
-        _counter+=1
+        if doc.page == 1:
+            _counter = 0
+            _counterStyle = 'arabic'
+        _counter += 1
 
         # Adjust text space accounting for header/footer
-        
+
         self.hh = self._head.prepare(self, canv, doc)
         self.fh = self._foot.prepare(self, canv, doc)
-            
+
         canv._doctemplate = doct
 
-        self.hx = self.styles.lm
-        self.hy = self.styles.ph - self.styles.tm -self.hh
+        self.hx = styles.lm
+        self.hy = styles.ph - styles.tm - self.hh
+
+        self.fx = styles.lm
+        self.fy = styles.bm
+        self.th = styles.ph - styles.tm - styles.bm - self.hh \
+                    - self.fh - styles.ts - styles.bs
 
-        self.fx = self.styles.lm
-        self.fy = self.styles.bm
-        self.th = self.styles.ph - self.styles.tm - \
-            self.styles.bm - self.hh - self.fh - \
-            self.styles.ts - self.styles.bs
-            
         # Adjust gutter margins
         if doc.page % 2: # Left page
-            x1 = self.styles.lm
+            x1 = styles.lm
         else: # Right page
-            x1 = self.styles.lm + self.styles.gm
-        y1 = self.styles.bm + self.fh + self.styles.bs
-        
+            x1 = styles.lm + styles.gm
+        y1 = styles.bm + self.fh + styles.bs
+
         # If there is a background parameter for this page Template, draw it
         if 'background' in self.template:
             self.draw_background('background', canv)
@@ -1087,13 +1063,13 @@ class FancyPage(PageTemplate):
         self.frames = []
         for frame in self.template['frames']:
             self.frames.append(SmartFrame(self,
-                self.styles.adjustUnits(frame[0], self.tw) + x1,
-                self.styles.adjustUnits(frame[1], self.th) + y1,
-                self.styles.adjustUnits(frame[2], self.tw),
-                self.styles.adjustUnits(frame[3], self.th),
+                styles.adjustUnits(frame[0], self.tw) + x1,
+                styles.adjustUnits(frame[1], self.th) + y1,
+                styles.adjustUnits(frame[2], self.tw),
+                styles.adjustUnits(frame[3], self.th),
                     showBoundary=self.show_frame))
-        canv.firstSect=True
-        canv._pagenum=doc.page
+        canv.firstSect = True
+        canv._pagenum = doc.page
         for frame in self.frames:
             frame._pagenum=doc.page
 
@@ -1101,7 +1077,7 @@ class FancyPage(PageTemplate):
         """Draw header/footer."""
         # Adjust for gutter margin
         canv.addPageLabel(canv._pageNumber-1,numberingstyles[_counterStyle],_counter)
-        
+
         log.info('Page %s [%s]'%(_counter,doc.page))
         if doc.page % 2: # Left page
             hx = self.hx
@@ -1109,7 +1085,7 @@ class FancyPage(PageTemplate):
         else: # Right Page
             hx = self.hx + self.styles.gm
             fx = self.fx + self.styles.gm
-            
+
         self._head.draw(self, canv, doc, hx, self.hy, self.tw, self.hh)
         self._foot.draw(self, canv, doc, fx, self.fy, self.tw, self.fh)
 
@@ -1119,12 +1095,12 @@ class FancyPage(PageTemplate):
 
 
 def parse_commandline():
-    
+
     parser = OptionParser()
-    
+
     parser.add_option('--config', dest='configfile', metavar='FILE',
         help='Config file to use. Default=~/.rst2pdf/config')
-    
+
     parser.add_option('-o', '--output', dest='output', metavar='FILE',
         help='Write the PDF to FILE')
 
@@ -1133,16 +1109,16 @@ def parse_commandline():
     parser.add_option('-s', '--stylesheets', dest='style',
         type='string', action='append',
         metavar='STYLESHEETS', default=[def_ssheets],
-        help='A comma-separated list of custom stylesheets.'\
-        ' Default="%s"' % def_ssheets)
+        help='A comma-separated list of custom stylesheets. Default="%s"'
+            % def_ssheets)
 
     def_sheetpath = os.pathsep.join([expanduser(p) for p in
         config.getValue("general", "stylesheet_path", "").split(os.pathsep)])
     parser.add_option('--stylesheet-path', dest='stylepath',
         metavar='FOLDER%sFOLDER%s...%sFOLDER'%((os.pathsep, )*3),
         default=def_sheetpath,
-        help='A list of folders to search for stylesheets,"\
-        " separated using "%s". Default="%s"' %(os.pathsep, def_sheetpath))
+        help='A list of folders to search for stylesheets,'
+            ' separated using "%s". Default="%s"' %(os.pathsep, def_sheetpath))
 
     def_compressed = config.getValue("general", "compressed", False)
     parser.add_option('-c', '--compressed', dest='compressed',
@@ -1161,59 +1137,69 @@ def parse_commandline():
     parser.add_option('--font-path', dest='fpath',
         metavar='FOLDER%sFOLDER%s...%sFOLDER'%((os.pathsep, )*3),
         default=def_fontpath,
-        help='A list of folders to search for fonts,'\
-             ' separated using "%s". Default="%s"'%(os.pathsep, def_fontpath))
+        help='A list of folders to search for fonts, separated using "%s".'
+            ' Default="%s"' % (os.pathsep, def_fontpath))
 
     def_baseurl = urlunparse(['file',os.getcwd()+os.sep,'','','',''])
     parser.add_option('--baseurl', dest='baseurl', metavar='URL',
         default=def_baseurl,
         help='The base URL for relative URLs. Default="%s"'%def_baseurl)
 
-    def_lang = config.getValue("general", "language", 'en')
+    def_lang = config.getValue("general", "language", 'en_US')
     parser.add_option('-l', '--language', metavar='LANG',
         default=def_lang, dest='language',
-        help='Language to be used for hyphenation and '\
-        'docutils localizations. Default="%s"' % def_lang)
+        help='Language to be used for hyphenation'
+            ' and docutils localizations. Default="%s"' % def_lang)
 
     def_header = config.getValue("general", "header")
     parser.add_option('--header', metavar='HEADER',
         default=def_header, dest='header',
-        help='Page header if not specified in the document.'\
-        ' Default="%s"' % def_header)
+        help='Page header if not specified in the document.'
+            ' Default="%s"' % def_header)
 
     def_footer = config.getValue("general", "footer")
     parser.add_option('--footer', metavar='FOOTER',
         default=def_footer, dest='footer',
-        help='Page footer if not specified in the document.'\
-        ' Default="%s"' % def_footer)
+        help='Page footer if not specified in the document.'
+            ' Default="%s"' % def_footer)
+
+    def_section_header_depth = config.getValue("general","section_header_depth",2)
+    parser.add_option('--section-header-depth', metavar='N',
+        default=def_section_header_depth, dest='section_header_depth',
+        help = '''Sections up to this depth will be used in the header and footer's replacement of ###Section###. Default=%s''' % def_section_header_depth)
 
     def_smartquotes = config.getValue("general", "smartquotes", "0")
     parser.add_option("--smart-quotes", metavar="VALUE",
         default=def_smartquotes, dest="smarty",
-        help='Try to convert ASCII quotes, ellipsis and dashes '\
-        'to the typographically correct equivalent. For details,'\
-        ' read the man page or the manual. Default="%s"'%def_smartquotes)
+        help='Try to convert ASCII quotes, ellipses and dashes'
+            ' to the typographically correct equivalent. For details,'
+            ' read the man page or the manual. Default="%s"' % def_smartquotes)
 
     def_fit = config.getValue("general", "fit_mode", "shrink")
     parser.add_option('--fit-literal-mode', metavar='MODE',
         default=def_fit, dest='fit_mode',
-        help='What todo when a literal is too wide. One of error,'\
-        ' overflow,shrink,truncate. Default="%s"'%def_fit)
+        help='What to do when a literal is too wide. One of error,'
+            ' overflow,shrink,truncate. Default="%s"' % def_fit)
 
     def_fit_background = config.getValue("general", "background_fit_mode",
-    "center")
+       "center")
     parser.add_option('--fit-background-mode', metavar='MODE',
         default=def_fit_background, dest='background_fit_mode',
-        help='How to fit the background image to the page.'\
-        ' One of scale or center. Default="%s"'%def_fit_background)
+        help='How to fit the background image to the page.'
+            ' One of scale or center. Default="%s"' % def_fit_background)
 
     parser.add_option('--inline-links', action="store_true",
     dest='inlinelinks', default=False,
-        help='shows target between parenthesis instead of active link')
+        help='Shows target between parentheses instead of active link.')
 
     parser.add_option('--repeat-table-rows', action="store_true",
         dest='repeattablerows', default=False,
-        help='Repeats header row for each splitted table')
+        help='Repeats header row for each split table.')
+
+    def_raw_html = config.getValue("general", "raw_html", False)
+    parser.add_option('--raw-html', action="store_true",
+        dest='raw_html', default=def_raw_html,
+        help='Support embeddig raw HTML. Default=%s' % def_raw_html)
 
     parser.add_option('-q', '--quiet', action="store_true",
         dest='quiet', default=False,
@@ -1235,22 +1221,22 @@ def parse_commandline():
         "footnote_backlinks", True)
     parser.add_option('--no-footnote-backlinks', action='store_false',
         dest='footnote_backlinks', default=def_footnote_backlinks,
-        help='Disable footnote backlinks.'\
-        ' Default=%s' % str(not def_footnote_backlinks))
+        help='Disable footnote backlinks.'
+            ' Default=%s' % str(not def_footnote_backlinks))
 
     def_inline_footnotes = config.getValue("general",
         "inline_footnotes", False)
     parser.add_option('--inline-footnotes', action='store_true',
         dest='inline_footnotes', default=def_inline_footnotes,
-        help='Show footnotes inline.'\
-        ' Default=%s' % str(not def_inline_footnotes))
+        help='Show footnotes inline.'
+            ' Default=%s' % str(not def_inline_footnotes))
 
     def_real_footnotes = config.getValue("general",
         "real_footnotes", False)
     parser.add_option('--real-footnotes', action='store_true',
         dest='real_footnotes', default=def_real_footnotes,
-        help='Show footnotes at the bottom of the page where they are defined.'\
-        ' Default=%s' % str(def_real_footnotes))
+        help='Show footnotes at the bottom of the page where they are defined.'
+            ' Default=%s' % str(def_real_footnotes))
 
     def_dpi = config.getValue("general", "default_dpi", 300)
     parser.add_option('--default-dpi', dest='def_dpi', metavar='NUMBER',
@@ -1263,20 +1249,20 @@ def parse_commandline():
 
     parser.add_option('--disable-splittables', dest='splittables',
         action='store_false', default=True,
-        help='Don\'t use splittable flowables in some elements. '
-        'Only try this if you can\'t process a document any other way.')
+        help="Don't use splittable flowables in some elements."
+            " Only try this if you can't process a document any other way.")
 
     def_break = config.getValue("general", "break_level", 0)
     parser.add_option('-b', '--break-level', dest='breaklevel',
         metavar='LEVEL', default=def_break,
-        help='Maximum section level that starts in a new page.'\
-        ' Default: %d' % def_break)
+        help='Maximum section level that starts in a new page.'
+            ' Default: %d' % def_break)
 
     def_fpeven = config.getValue("general", "first_page_even", False)
     parser.add_option('--first-page-even', dest='first_page_even',
         action='store_true', default=def_fpeven,
-        help='Whether first page is odd (as in the screen on "facing pages"), '\
-             'or even (as in a book)')
+        help='Whether first page is odd (as in the screen on "facing pages"),'
+            ' or even (as in a book).')
 
     def_blankfirst = config.getValue("general", "blank_first_page", False)
     parser.add_option('--blank-first-page', dest='blank_first_page',
@@ -1286,14 +1272,16 @@ def parse_commandline():
     def_breakside = config.getValue("general", "break_side", 'any')
     parser.add_option('--break-side', dest='breakside', metavar='VALUE',
         default=def_breakside,
-        help='How section breaks work. Can be "even", and sections start in an even page,'\
-        '"odd", and sections start in odd pages, or "any" and sections start in the next page,'\
-        'be it even or odd. See also the -b option.')
+        help='How section breaks work. Can be "even", and sections start'
+            ' in an even page, "odd", and sections start in odd pages,'
+            ' or "any" and sections start in the next page, be it even or odd.'
+            ' See also the -b option.')
 
-    parser.add_option('--date-invariant', dest='invariant', 
+    parser.add_option('--date-invariant', dest='invariant',
         action='store_true', default=False,
-        help="Don't store the current date in the PDF. Useful mainly for the test suite, "\
-        "where we don't want the PDFs to change.")
+        help="Don't store the current date in the PDF."
+            " Useful mainly for the test suite,"
+            " where we don't want the PDFs to change.")
 
     parser.add_option('-e', '--extension-module', dest='extensions', action="append", type="string",
         default = ['vectorpdf'],
@@ -1307,7 +1295,7 @@ def parse_commandline():
 
     def_floating_images = config.getValue("general", "floating_images", False)
     parser.add_option('--use-floating-images', action='store_true', default=def_floating_images,
-        help='Makes images with :aling: attribute work more like in rst2html. Default: %s'%def_floating_images,
+        help='Makes images with :align: attribute work more like in rst2html. Default: %s'%def_floating_images,
         dest='floating_images')
 
     def_numbered_links = config.getValue("general", "numbered_links", False)
@@ -1317,16 +1305,20 @@ def parse_commandline():
 
     return parser
 
-def main(args=None):
+def main(_args=None):
     """Parse command line and call createPdf with the correct data."""
 
     parser = parse_commandline()
-    options, args = parser.parse_args(copy(args))
+    # Fix issue 430: don't overwrite args
+    # need to parse_args to see i we have a custom config file
+    options, args = parser.parse_args(copy(_args))
 
     if options.configfile:
-        options.cfname=options.configfile
+        # If there is a config file, we need to reparse
+        # the command line because we have different defaults
+        config.parseConfig(options.configfile)
         parser = parse_commandline()
-        options, args = parser.parse_args(copy(args))
+        options, args = parser.parse_args(copy(_args))
 
     if options.version:
         from rst2pdf import version
@@ -1415,9 +1407,8 @@ def main(args=None):
         options.inline_footnotes = True
 
     if reportlab.Version < '2.3':
-        log.warning('You are using Reportlab version %s.'\
-            ' The suggested version '\
-            'is 2.3 or higher'%reportlab.Version)
+        log.warning('You are using Reportlab version %s.'
+            ' The suggested version is 2.3 or higher' % reportlab.Version)
 
     if options.invariant:
         patch_PDFDate()
@@ -1450,6 +1441,8 @@ def main(args=None):
         custom_cover=options.custom_cover,
         floating_images=options.floating_images,
         numbered_links=options.numbered_links,
+        raw_html=options.raw_html,
+	    section_header_depth=int(options.section_header_depth),
         ).createPdf(text=options.infile.read(),
                     source_path=options.infile.name,
                     output=options.outfile,
@@ -1489,7 +1482,7 @@ def patch_PDFDate():
                     lambda yyyy,mm,dd,hh,m,s:
                         "D:%04d%02d%02d%02d%02d%02d%+03d'%02d'" % (yyyy,mm,dd,hh,m,s,0,0))
             return pdfdoc.format(pdfdoc.PDFString(dfmt(*self.date)), doc)
-        
+
     pdfdoc.PDFDate = PDFDate
     reportlab.rl_config.invariant = 1
 
