@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 # See LICENSE.txt for licensing terms
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/extensions/dotted_toc.py $
-#$LastChangedDate: 2010-03-18 15:51:01 -0300 (Thu, 18 Mar 2010) $
-#$LastChangedRevision: 1944 $
+#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/extensions/dotted_toc.py $
+#$LastChangedDate: 2010-11-30 18:39:28 -0300 (Tue, 30 Nov 2010) $
+#$LastChangedRevision: 2417 $
 
 # Some fragments of code are copied from Reportlab under this license:
 #
@@ -132,7 +132,7 @@ class DottedTableOfContents(genelements.MyTableOfContents):
 
             style = copy(style)
             style.textColor = self.linkColor
-            key = self.refid_lut.get((level, text), None)
+            key = self.refid_lut.get((level, text, pageNum), None)
             if key:
                 if not isinstance(text, unicode):
                     text = unicode(text, 'utf-8')
