@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # See LICENSE.txt for licensing terms
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/flowables.py $
-#$LastChangedDate: 2010-10-06 17:17:48 -0300 (Wed, 06 Oct 2010) $
-#$LastChangedRevision: 2393 $
+#$URL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/flowables.py $
+#$Date: 2012-02-28 21:07:21 -0300 (Tue, 28 Feb 2012) $
+#$Revision: 2443 $
 
 __docformat__ = 'reStructuredText'
 
@@ -56,7 +56,7 @@ class XXPreformatted(XPreformatted):
             pw, ph = self.wrap(aW, aH)
             if ph - aH < minH2:
                 aH = ph - minH2
-            
+
         return XPreformatted.split(self, aW, aH)
 
 class MyIndenter(Indenter):
@@ -90,12 +90,7 @@ class Heading(Paragraph):
     the PDF TOC."""
 
     def __init__(self, text, style, bulletText=None, caseSensitive=1, level=0,
-                 snum=None, parent_id=None, node=None):
-        #if label is None: # it happens
-            #self.label = text.replace(u'\xa0', ' ').strip(
-                #).replace(' ', '-').encode('ascii', 'replace').lower()
-        #else:
-            #self.label = label.strip()
+                 snum=None, parent_id=None, node=None, section_header_depth=2):
         # Issue 114: need to convert "&amp;" to "&" and such.
         # Issue 140: need to make it plain text
         self.stext=re.sub(r'<[^>]*?>', '', unescape(text))
@@ -104,13 +99,15 @@ class Heading(Paragraph):
         self.snum = snum
         self.parent_id=parent_id
         self.node=node
+        self.section_header_depth = section_header_depth
         Paragraph.__init__(self, text, style, bulletText)
 
     def draw(self):
 
         # Add outline entry
         self.canv.bookmarkHorizontal(self.parent_id,0,0+self.height)
-        if self.canv.firstSect:
+        # self.section_header_depth is for Issue 391
+        if self.canv.firstSect and self.level < self.section_header_depth:
             self.canv.sectName = self.stext
             self.canv.firstSect=False
             if self.snum is not None:
@@ -164,7 +161,7 @@ class OddEven(Flowable):
     wrap() will always return a size large enough for both lists, and this flowable
     **cannot** be split, so use with care.
     """
-    
+
     def __init__(self, odd, even, style=None):
         self.odd=DelayedTable([[odd]],['100%'], style)
         self.even=DelayedTable([[even]],['100%'], style)
@@ -213,7 +210,7 @@ class DelayedTable(Table):
         #self.repeatRows= 0
         #self.splitByRow= 1
         #self.vAlign= 'MIDDLE'
-        
+
     def wrap(self, w, h):
         # Create the table, with the widths from colWidths reinterpreted
         # if needed as percentages of frame/cell/whatever width w is.
@@ -226,7 +223,7 @@ class DelayedTable(Table):
         self.colWidths=map(adjust, self._colWidths)
         #colWidths = [_w * _tw for _w in self.colWidths]
         self.t = Table(self.data, colWidths=self.colWidths,
-            style=self.style, repeatRows=self.repeatrows, 
+            style=self.style, repeatRows=self.repeatrows,
             splitByRow=True)
             #splitByRow=self.splitByRow)
         self.t.hAlign = self.hAlign
@@ -242,7 +239,7 @@ class DelayedTable(Table):
 
     def drawOn(self, canvas, x, y, _sW=0):
         self.t.drawOn(canvas, x, y, _sW)
-        
+
     def identity(self, maxLen=None):
         return "<%s at %s%s%s> containing: %s" % (self.__class__.__name__,
             hex(id(self)), self._frameName(),
@@ -302,7 +299,7 @@ class SplitTable(DelayedTable):
                     if l2==[]: # Not splittable, push some to next page
                         if l==0: # Can't fit anything, push all to next page
                             return l2
- 
+
                         # We reduce the number of items we keep on the
                         # page for two reasons:
                         #    1) If an item is associated with the following
@@ -380,7 +377,7 @@ class MySpacer(Spacer):
         self.height = min(aH, h)
         return w, self.height
 
-    
+
 
 class MyPageBreak(FrameActionFlowable):
 
@@ -688,12 +685,13 @@ class BoundByWidth(Flowable):
 
     """
 
-    def __init__(self, maxWidth, content=[], style=None, mode=None):
+    def __init__(self, maxWidth, content=[], style=None, mode=None, scale = None):
         self.maxWidth = maxWidth
         self.content = content
         self.style = style
         self.mode = mode
         self.pad = None
+        self.scale = scale
         Flowable.__init__(self)
 
     def border_padding(self, useWidth, additional):
@@ -722,15 +720,18 @@ class BoundByWidth(Flowable):
         self.maxWidth = maxWidth
         maxWidth -= (self.pad[1]+self.pad[3])
         self.width, self.height = _listWrapOn(self.content, maxWidth, None)
-        self.scale = 1.0
         if self.width > maxWidth:
             if self.mode <> 'shrink':
+                self.scale = 1.0
                 log.warning("BoundByWidth too wide to fit in frame (%s > %s): %s",
                     self.width,maxWidth,self.identity())
-            if self.mode == 'shrink':
+            if self.mode == 'shrink' and not self.scale:
                 self.scale = (maxWidth + self.pad[1]+self.pad[3])/\
                     (self.width + self.pad[1]+self.pad[3])
-                self.height *= self.scale
+        else:
+            self.scale = 1.0
+        self.height *= self.scale
+        self.width *= self.scale
         return self.width, self.height + (self.pad[0]+self.pad[2])*self.scale
 
     def split(self, availWidth, availHeight):
@@ -740,10 +741,11 @@ class BoundByWidth(Flowable):
         if len(self.content) == 1:
             # We need to split the only element we have
             content = content[0].split(
-                availWidth - (self.pad[1]+self.pad[3]),
-                availHeight - (self.pad[0]+self.pad[2]))
-        return [BoundByWidth(self.maxWidth, [f],
-                             self.style, self.mode) for f in content]
+                (1./self.scale) * (availWidth - (self.pad[1]+self.pad[3])),
+                (1./self.scale) * (availHeight - (self.pad[0]+self.pad[2])))
+        result =  [BoundByWidth(self.maxWidth, [f],
+                             self.style, self.mode, self.scale) for f in content]
+        return result
 
     def draw(self):
         """we simulate being added to a frame"""
@@ -914,18 +916,18 @@ class MyTableOfContents(TableOfContents):
     """
 
     def __init__(self, *args, **kwargs):
-        
+
         # The parent argument is to define the locality of
         # the TOC. If it's none, it's a global TOC and
         # any heading it's notified about is accepted.
-        
+
         # If it's a node, then the heading needs to be "inside"
         # that node. This can be figured out because
         # the heading flowable keeps a reference to the title
         # node it was creatd from.
         #
         # Yes, this is gross.
-        
+
         self.parent=kwargs.pop('parent')
         TableOfContents.__init__(self, *args, **kwargs)
         # reference ids for which this TOC should be notified
@@ -938,7 +940,7 @@ class MyTableOfContents(TableOfContents):
         # stuff includes (level, text, pagenum, label)
         level, text, pageNum, label, node = stuff
         rlabel='-'.join(label.split('-')[:-1])
-        
+
         def islocal(_node):
             '''See if this node is "local enough" for this TOC.
             This is for Issue 196'''
@@ -949,7 +951,7 @@ class MyTableOfContents(TableOfContents):
                     return True
                 _node=_node.parent
             return False
-            
+
         if rlabel in self.refids and islocal(node):
             self.addEntry(level, text, pageNum)
             self.refid_lut[(level, text, pageNum)] = label
