@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/genelements.py $
-#$LastChangedDate: 2010-10-06 17:17:48 -0300 (Wed, 06 Oct 2010) $
-#$LastChangedRevision: 2393 $
+#$URL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/genelements.py $
+#$Date: 2012-05-25 23:11:53 -0300 (Fri, 25 May 2012) $
+#$Revision: 2503 $
 
 # See LICENSE.txt for licensing terms
 
@@ -53,14 +53,15 @@ import reportlab
 from aafigure_directive import Aanode
 
 from log import log, nodeid
-from utils import log, parseRaw
+from utils import log, parseRaw, parseHTML
 from reportlab.platypus import Paragraph, TableStyle
 from reportlab.lib.units import cm
 from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
 from flowables import Table, DelayedTable, SplitTable, Heading, \
               MyIndenter, MyTableOfContents, MySpacer, \
               Separation, BoxedContainer, BoundByWidth, \
-              MyPageBreak, Reference, tablepadding, OddEven
+              MyPageBreak, Reference, tablepadding, OddEven, \
+              XPreformatted
 
 from opt_imports import wordaxe, Paragraph, ParagraphStyle
 
@@ -100,7 +101,7 @@ class HandleTGroup(NodeHandler, docutils.nodes.tgroup):
 
         # Take the style from the parent "table" node
         # because sometimes it's not passed down.
-        
+
         if node.parent['classes']:
             style = client.styles.combinedStyle(['table']+node.parent['classes'])
         else:
@@ -152,7 +153,7 @@ class HandleTGroup(NodeHandler, docutils.nodes.tgroup):
                         st = client.styles['table-heading']
                     else:
                         st = client.styles['table-body']
-                    ell = client.gather_elements(cell, style=st)                    
+                    ell = client.gather_elements(cell, style=st)
                     r.append(ell)
                 j += 1
             data.append(r)
@@ -164,15 +165,15 @@ class HandleTGroup(NodeHandler, docutils.nodes.tgroup):
         else:
             # Only use the commands from "table" if the
             # specified class has no commands.
-            
+
             for cmd in client.styles['table'].commands:
                 st.add(*cmd)
-            
+
         if hasHead:
             for cmd in client.styles.tstyleHead(headRows):
                 st.add(*cmd)
         rtr = client.repeat_table_rows
-        
+
         t=DelayedTable(data, colWidths, st, rtr)
         if style.alignment == TA_LEFT:
             t.hAlign='LEFT'
@@ -232,14 +233,15 @@ class HandleTitle(HandleParagraph, docutils.nodes.title):
             maxdepth=4
             if reportlab.Version > '2.1':
                 maxdepth=6
-                
+
             # The parent ID is the refid + an ID to make it unique for Sphinx
             parent_id=(node.parent.get('ids', [None]) or [None])[0]+u'-'+unicode(id(node))
             node.elements = [ Heading(text,
                     client.styles['heading%d'%min(client.depth, maxdepth)],
                     level=client.depth-1,
                     parent_id=parent_id,
-                    node=node
+                    node=node,
+                    section_header_depth=client.section_header_depth
                     )]
             if client.depth <= client.breaklevel:
                 node.elements.insert(0, MyPageBreak(breakTo=client.breakside))
@@ -359,6 +361,17 @@ class HandleContact(HandleFList, docutils.nodes.contact):
 
 class HandleAddress(HandleFList, docutils.nodes.address):
     labeltext = "address"
+    def gather_elements(self, client, node, style):
+        fb = client.gather_pdftext(node)
+        t_style=TableStyle(client.styles['field-list'].commands)
+        colWidths=client.styles['field-list'].colWidths
+        if self.adjustwidths:
+            colWidths = map(client.styles.adjustUnits, colWidths)
+        label=client.text_for_label(self.labeltext, style)+":"
+        t = self.TableType([[Paragraph(label, style=client.styles['fieldname']),
+                             XPreformatted(fb, style)]
+                    ], style=t_style, colWidths=colWidths)
+        return [t]
 
 class HandleVersion(HandleFList, docutils.nodes.version):
     labeltext = "version"
@@ -432,18 +445,18 @@ class HandleSection(NodeHandler, docutils.nodes.section):
 
 class HandleBulletList(NodeHandler, docutils.nodes.bullet_list):
     def gather_elements(self, client, node, style):
-        
+
         if node ['classes']:
             style = client.styles[node['classes'][0]]
         else:
             style = client.styles["bullet-list"]
-            
+
         node.elements = client.gather_elements(node,
             style=style)
-        
+
         # Here we need to separate the list from the previous element.
         # Calculate by how much:
-        
+
         sb=style.spaceBefore # list separation
         sa=style.spaceAfter # list separation
 
@@ -466,13 +479,13 @@ class HandleEnumeratedList(NodeHandler, docutils.nodes.enumerated_list):
             style = client.styles[node['classes'][0]]
         else:
             style = client.styles["item-list"]
-        
+
         node.elements = client.gather_elements(node,
             style = style)
-        
+
         # Here we need to separate the list from the previous element.
         # Calculate by how much:
-        
+
         sb=style.spaceBefore # list separation
         sa=style.spaceAfter # list separation
 
@@ -483,7 +496,7 @@ class HandleEnumeratedList(NodeHandler, docutils.nodes.enumerated_list):
 class HandleDefinition(NodeHandler, docutils.nodes.definition):
     def gather_elements(self, client, node, style):
         return client.gather_elements(node,
-                       style = client.styles["definition"])
+                       style = style)
 
 class HandleOptionListItem(NodeHandler, docutils.nodes.option_list_item):
     def gather_elements(self, client, node, style):
@@ -522,7 +535,7 @@ class HandleDefListItem(NodeHandler, docutils.nodes.definition_list_item):
         # FIXME: make this configurable from the stylesheet
         t_style = TableStyle (client.styles['definition'].commands)
         cw = getattr(client.styles['definition'],'colWidths',[])
-        
+
         if client.splittables:
             node.elements = [
                 Paragraph(''.join(ids)+' : '.join(tt), client.styles['definition-list-term']),
@@ -531,7 +544,7 @@ class HandleDefListItem(NodeHandler, docutils.nodes.definition_list_item):
             node.elements = [
                 Paragraph(''.join(ids)+' : '.join(tt), client.styles['definition-list-term']),
                 DelayedTable([['',dt]] , colWidths=[10,None], style = t_style )]
-            
+
         return node.elements
 
 class HandleListItem(NodeHandler, docutils.nodes.list_item):
@@ -551,7 +564,7 @@ class HandleListItem(NodeHandler, docutils.nodes.list_item):
         # bulletFont
         # This is so the baselines of the bullet and the text align
         extra_space= bStyle.bulletFontSize-bStyle.fontSize
-        
+
         bStyle.fontSize=bStyle.bulletFontSize
         bStyle.fontName=bStyle.bulletFontName
 
@@ -595,12 +608,12 @@ class HandleListItem(NodeHandler, docutils.nodes.list_item):
         else:
             # The bullet is smaller, move down the bullet
             sbb = -extra_space
-            
+
         #colWidths = map(client.styles.adjustUnits,
             #client.styles['item_list'].colWidths)
         colWidths = getattr(style,'colWidths',[])
         while len(colWidths) < 2:
-            colWidths.append(None)
+            colWidths.append(client.styles['item_list'].colWidths[len(colWidths)])
 
         if client.splittables:
             node.elements = [MySpacer(0,sb),
@@ -675,7 +688,7 @@ class HandleLineBlock(NodeHandler, docutils.nodes.line_block):
         # Fix Issue 225: no space betwen line in a lineblock, but keep
         # space before the lineblock itself
         return [MySpacer(0,client.styles['lineblock'].spaceBefore)]+client.gather_elements(node, style=qstyle)+[MySpacer(0,client.styles['lineblock'].spaceAfter)]
-        
+
 class HandleLine(NodeHandler, docutils.nodes.line):
     def gather_elements(self, client, node, style):
         # Indent .5em per indent unit
@@ -703,7 +716,7 @@ class HandleLiteralBlock(NodeHandler, docutils.nodes.literal_block,
 class HandleFigure(NodeHandler, docutils.nodes.figure):
     def gather_elements(self, client, node, style):
 
-        # Either use the figure style or the class 
+        # Either use the figure style or the class
         # selected by the user
         if node.get('classes'):
             style=client.styles[node.get('classes')[0]]
@@ -728,7 +741,7 @@ class HandleFigure(NodeHandler, docutils.nodes.figure):
         table.hAlign = node.get('align','CENTER').upper()
         return [table]
 
-            
+
 class HandleCaption(NodeHandler, docutils.nodes.caption):
     def gather_elements(self, client, node, style):
         return [Paragraph(client.gather_pdftext(node),
@@ -850,6 +863,9 @@ class HandleRaw(NodeHandler, docutils.nodes.raw):
         # Not really raw, but what the heck
         if node.get('format','NONE').lower()=='pdf':
             return parseRaw(str(node.astext()), node)
+        elif client.raw_html and node.get('format','NONE').lower()=='html':
+            x = parseHTML(str(node.astext()), node)
+            return x
         else:
             return []
 
