@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/genpdftext.py $
-#$LastChangedDate: 2010-10-06 17:17:48 -0300 (Wed, 06 Oct 2010) $
-#$LastChangedRevision: 2393 $
+#$URL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/genpdftext.py $
+#$Date: 2012-03-06 22:44:18 -0300 (Tue, 06 Mar 2012) $
+#$Revision: 2496 $
 
 # See LICENSE.txt for licensing terms
 
@@ -44,7 +44,7 @@ class HandleEmphasis(NodeHandler, docutils.nodes.emphasis):
 
 class HandleLiteral(NodeHandler, docutils.nodes.literal):
     def get_pre_post(self, client, node, replaceEnt):
-        
+
         if node['classes']:
             pre = client.styleToFont(node['classes'][0])
         else:
@@ -55,6 +55,12 @@ class HandleLiteral(NodeHandler, docutils.nodes.literal):
             post += '</nobr>'
         return pre, post
 
+    def get_text(self, client, node, replaceEnt):
+        text = node.astext()
+        text = escape(node.astext())
+        text = text.replace(' ', '&nbsp;')
+        return text
+        
 class HandleSuper(NodeHandler, docutils.nodes.superscript):
     pre = '<super>'
     post = "</super>"
@@ -136,7 +142,7 @@ class HandleImage(NodeHandler, docutils.nodes.image):
 
         try:
             w, h, kind = MyImage.size_for_node(node, client=client)
-        except ValueError: 
+        except ValueError:
             # Broken image, return arbitrary stuff
             imgname=missing
             w, h, kind = 100, 100, 'direct'
@@ -156,7 +162,7 @@ class HandleImage(NodeHandler, docutils.nodes.image):
         imgname = os.path.join(client.basedir,str(node.get("uri")))
         try:
             w, h, kind = MyImage.size_for_node(node, client=client)
-        except ValueError: 
+        except ValueError:
             # Broken image, return arbitrary stuff
             imgname=missing
             w, h, kind = 100, 100, 'direct'
