@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/sphinxnodes.py $
-#$LastChangedDate: 2010-10-06 17:17:48 -0300 (Wed, 06 Oct 2010) $
-#$LastChangedRevision: 2393 $
+#$URL: https://rst2pdf.googlecode.com/svn/tags/0.92/rst2pdf/sphinxnodes.py $
+#$Date: 2012-02-28 21:07:21 -0300 (Tue, 28 Feb 2012) $
+#$Revision: 2443 $
 
 # See LICENSE.txt for licensing terms
 
@@ -162,20 +162,23 @@ class HandleHList(SphinxHandler, sphinx.addnodes.hlist):
         # Each child is a hlistcol and represents a column.
         # Each grandchild is a bullet list that's the contents
         # of the column
-        
+
         # Represent it as a N-column, 1-row table, each cell containing
         # a list.
-        
+
         cells = [[ client.gather_elements(child, style) for child in node.children]]
-        t_style=TableStyle(client.styles['hlist'].commands)        
+        t_style=TableStyle(client.styles['hlist'].commands)
         cw=100./len(node.children)
-        return [ DelayedTable( cells, 
+        return [ DelayedTable( cells,
             colWidths=["%s%%"%cw,]*len(cells),
             style=t_style
             )]
 
 from sphinx.ext import mathbase
 
+class HandleHighlightLang(SphinxHandler, sphinx.addnodes.highlightlang):
+    pass
+
 class HandleSphinxMath(SphinxHandler, mathbase.math, mathbase.displaymath):
     def gather_elements(self, client, node, style):
         mflow=math_flowable.Math(node.get('latex',''),node.get('label',None))
@@ -197,7 +200,7 @@ class HandleSphinxMath(SphinxHandler, mathbase.math, mathbase.displaymath):
 class HandleSphinxEq(SphinxHandler, mathbase.eqref):
 
     def get_text(self, client, node, replaceEnt):
-        return '<a href="equation-%s" color="%s">%s</a>'%(node['target'], 
+        return '<a href="equation-%s" color="%s">%s</a>'%(node['target'],
             client.styles.linkColor, node.astext())
 
 graphviz_warn = False
@@ -233,7 +236,7 @@ try:
 except AttributeError:
     # Probably the graphviz extension is not enabled
     pass
-    
+
 
 
 sphinxhandlers = SphinxHandler()
